package server.command;

import java.lang.reflect.Method;
import java.util.List;
import java.util.Map;

import common.collect.Lists;
import common.collect.Maps;

public class CachedExecutable {
	private final Executable executable;
	private final String name;
	private final Map<String, Parameter> parameters;
	private final List<Parameter> positionals;
	private final Method method;
	
	protected CachedExecutable(Executable executable, Map<String, Parameter> parameters, List<Parameter> positionals, Method method) {
		this.executable = executable;
		this.parameters = parameters;
		this.positionals = positionals;
		this.name = executable.getName();
		this.method = method;
	}
	
	public static CachedExecutable cacheExecutable(Executable executable) {
		Map<String, Parameter> parameters = Maps.newTreeMap();
		Map<String, Parameter> params = executable.getParameters();
		parameters.putAll(params);
		Parameter[] positions = new Parameter[parameters.size()];
		int pos = -1;
		for(Parameter param : params.values()) {
			if(param.isPositional()) {
				positions[param.getPosition()] = param;
				pos = param.getPosition() > pos ? param.getPosition() : pos;
			}
			if(param.hasShortName())
				parameters.put("" + param.getShortName(), param);
		}
		List<Parameter> positionals = Lists.newArrayList();
		for(int z = 0; z <= pos; z++) {
			if(positions[z] == null)
				throw new NullPointerException("positions[" + z + "]");
			positionals.add(positions[z]);
		}
		List<Class<?>> classes = Lists.newArrayList(CommandEnvironment.class, Executor.class);
		for(Parameter param : executable.getParamList()) {
			ArgCombiner<?> combiner = param.getCombiner();
			if(combiner != null) {
				classes.add(combiner.getTypeClass());
				continue;
			}
			if(param.getParsers().isEmpty()) {
				classes.add(boolean.class);
				continue;
			}
			for(ArgumentParser parser : param.getParsers()) {
				classes.add(parser.getTypeClass());
			}
		}
		Method method;
		try {
			method = executable.getClass().getDeclaredMethod("exec", classes.toArray(new Class<?>[classes.size()]));
		}
		catch(NoSuchMethodException e) {
			throw new RuntimeException(e);
		}
		return new CachedExecutable(executable, parameters, positionals, method);
	}
	
	public Executable getExecutable() {
		return this.executable;
	}
	
	public Method getMethod() {
		return this.method;
	}
	
	public Map<String, Parameter> getParameters() {
		return this.parameters;
	}
	
	public List<Parameter> getPositionals() {
		return this.positionals;
	}
	
	public String getName() {
		return this.name;
	}
	
	public String toString() {
		return this.name;
	}
}
